require "defines"

local on_tick = defines.events.on_tick

local uminer = "underground-mining-drill"

local format = string.format

local root, floor, ceil, rand, min, max = math.sqrt, math.floor, math.ceil, math.random, math.min, math.max

local needcheck = true

local underground_of, shallow_of, minimum, mult = {}, {}, {}, {}

function setundergroundores(chunkey, resource, surface) --resource is name of resource (or all) or table=true of resources (empty set = all)
	surface = surface or chunkey.surface
	
	for i=chunkey.x-1,chunkey.x+1 do
		for j=chunkey.y-1,chunkey.y+1 do
			if not surface.is_chunk_generated{i,j} then return end
		end
	end
	
	--[[
	for _,p in pairs(game.players) do
		p.print("regenerating"..serpent.line(resource)..chunkey.x..chunkey.y)
	end
	--]]
	
	local uname, maxval
	local x,y = chunkey.x*32,chunkey.y*32
	
	if resource then --construct (underground) resource list
		uname=type(resource)
		if uname=="string" then
			if resource=="all" then
				resource = shallow_of
			else
				resource = {[underground_of[resource]]=true}
			end
		elseif uname=="table" then
			if not next(resource) then
				resource = shallow_of
			else
				for _,res in pairs(resource) do
					resource[underground_of[res]]=true
					resource[_]=nil
				end		
			end
		else
			return
		end
	else
		resource = shallow_of
	end
	
	--delete all flagged (perhaps unnecessary)
	for _,existing in pairs(surface.find_entities_filtered{area={{x,y},{x+32,y+32}},type="resource"}) do
		if resource[existing.name] then existing.destroy() end
	end
	
	local cache, indx ={}
	
	local rx,ry,qx,qy
	
	for _,r in pairs(surface.find_entities_filtered{area={{x-2,y-2},{x+36,y+36}},type="resource"}) do
		uname = underground_of[r.name]
		if uname then
			
			rx,ry = r.position.x,r.position.y
			qx,qy = floor((rx-2)/4)*4, floor((ry-2)/4)*4	--upperleft()
			rx,ry = floor(rx/4)*4, floor(ry/4)*4

			for tx = max(qx,x),min(qx+4,x+28),4 do
				for ty = max(qy,y),min(qy+4,y+28),4 do
					indx = tx/4+ty/4*8
					if not cache[indx] then cache[indx] = {r={},p={tx+2.01+rand()*.4,ty+2.01+rand()*.4}} end
					cache[indx].r[uname] = (cache[indx].r[uname] or 0) + r.amount *r.amount *1/ (1+((tx-rx)*(tx-rx)+(ty-ry)*(ty-ry))/4)
				end
			end
		end
	end
	
	for _,c in pairs(cache) do
		maxval=0
		for r,v in pairs(c.r) do
			qx = minimum[r]
			v = v/qx/qx
			if v > maxval then
				maxval = v
				uname = r
			end
		end
		
		if resource[uname] then
			maxval = ceil(root(c.r[uname]/16 * mult[uname]))
			if minimum[uname]*2 <= maxval and surface.can_place_entity{name=uminer, position=c.p} then
				indx = surface.create_entity{name=uname,position=c.p}
				indx.amount = maxval
			end
		end
	end	
end

function regenerate(resource, full)
	if not underground_of[resource] then return end
	local existing={}
	if full then
		for _,surface in pairs(game.surfaces) do
			for ch in surface.get_chunks() do
				for _,res in pairs(surface.find_entities_filtered{area={{ch.x*32,ch.y*32},{ch.x*32+32,ch.y*32+32}},name=resource}) do
					table.insert(existing,{name=res.name,position=res.position,amount=res.amount,surf=surface})
--					res.destroy()
				end
			end
		end
		game.regenerate_entity(resource)
	end
	for _,surface in pairs(game.surfaces) do
		for ch in surface.get_chunks() do
			setundergroundores(ch,resource, surface)
		end
		if full then
			for ch in surface.get_chunks() do
				for _,res in pairs(surface.find_entities_filtered{area={{ch.x*32,ch.y*32},{ch.x*32+32,ch.y*32+32}},name=resource}) do
					res.destroy()
				end
			end
		end
	end
	if full then
		for _,e in pairs(existing) do
			e.surf.create_entity{name=e.name,position=e.position}.amount = e.amount
		end
	end
	global.handled[resource]=true
end

function execute_queued()--this is the main tick executor
	if needcheck and game.tick>1 then
		if next(global.handled) then
			for res in pairs(underground_of) do
				if not global.handled[res] then regenerate(res) end
			end
		else
			for _,surf in pairs(game.surfaces) do
				for ch in surf.get_chunks() do
					ch.surface = surf
					setundergroundores(ch)
				end
				for res in pairs(underground_of) do
					global.handled[res]=true
				end
			end
		end
		
		script.on_event(defines.events.on_chunk_generated, function(event)
			local surface = event.surface
			local x, y = floor(event.area.left_top.x/32), floor(event.area.left_top.y/32)

			for i=x-1,x+1 do
				for j=y-1,y+1 do
					table.insert(global.chunqueue,{x=i,y=j,surface=surface})
				end
			end
			--game.get_player(1).print("chunk generated "..x..","..y)
			script.on_event(on_tick, execute_queued)
		end)
		
		needcheck = false
		
		return
	end

	for _,chunkey in pairs(global.chunqueue) do	--newstuff
		setundergroundores(chunkey)
	end
	global.chunqueue={}
	if not needcheck then script.on_event(on_tick, nil) end
end

function initialize()
	local uname, proto
	if not global.handled then global.handled = {} end
	if not global.chunqueue then global.chunqueue = {} end
	for _,e in pairs(game.entity_prototypes) do
		if e.type=="resource" then
			uname = "underground-"..e.name
			proto = game.entity_prototypes[uname]
			if proto then
				underground_of[e.name]=uname
				shallow_of[uname]=e.name
				if proto.infinite_resource then mult[uname]=12 else mult[uname]=26 end
				minimum[uname] = proto.minimum_resource_amount or 1
			end
		end
	end
	
	for res in pairs(global.handled) do
		if not underground_of[res] then global.handled[res]=nil end
	end

	script.on_event(on_tick, execute_queued)
end

script.on_event(on_tick, initialize)
--script.on_configuration_changed(initialize)--shouldn't be necessary

remote.add_interface("underground-mining",{
	regenerate=function(res)
		if not res then
			res=game.get_player(1).selected
			if not res then return end
			res=res.name
		end
		if res=="all" then
			for res in pairs(underground_of) do
				regenerate(res, true)
			end
		else
			regenerate(res, true)
		end
	end,
	refresh=function(res)
		if not res then
			res=game.get_player(1).selected
			if res then
				res=res.name
				if not underground_of[res] then res=nil end
			end
		end
		for _,surface in pairs(game.surfaces) do
			for ch in surface.get_chunks() do
				setundergroundores(ch,res, surface)
			end
		end
	end,
	global=function()
		game.get_player(1).print(serpent.line(global))
	end
})

--[[
script.on_event(defines.events.on_player_created, function(event)
game.get_player(event.player_index).insert{name="underground-mining-drill",count=12}
end)
--]]
