require "util"
local deepcopy = util.table.deepcopy

local logger={"\nUnderground mining log"..serpent.block(data.modcom.underground_mining)}
local root, floor, ceil, log10 = math.sqrt, math.floor, math.ceil, math.log10

local be_finite = data.modcom.underground_mining.finite
local pure = data.modcom.underground_mining.pure
local defined_richness = data.modcom.underground_mining.richness
local ignore = data.modcom.underground_mining.do_not_calculate

function makenice(value)
	local e = 10^floor(log10(value))
	if e > 1 then e=e/2 end
	return ceil(value/e)*e
end

local yieldresults = 1

local normal, stages, temp
local newres,ures={}
for _,res in pairs(data.raw.resource) do
	if (not res.category or res.category=="basic-solid") and not res.infinite and res.autoplace and not ignore[res.name] then
		ures = deepcopy(res)
		
		ures.name = "underground-"..ures.name
		ures.flags = {"placeable-off-grid"}
		ures.category = "underground-solid"
		ures.collision_box={{-.01,-.01},{.01,.01}}--{{0,0},{0,0}}
		ures.selection_box={{0,0},{0,0}}
--		ures.selection_box={{-.4,-.4},{.4,.4}}
		ures.stages.sheet.tint={r=.1875,g=.1875,b=.1875,a=.234375}--{r=.125,g=.125,b=.125,a=.21875}
		ures.stages.sheet.scale=4.25
		ures.map_color.a=.05
		
		ures.minable.mining_time=ures.minable.mining_time*1.5
		
		if not pure[res.name] then
			if ures.minable.result then
				ures.minable.results = {{type="item",name=ures.minable.result,probability=((ures.minable.probability or 1) * .875),amount=ures.minable.result.amount or 1}}
				ures.minable.result=nil
			end
			table.insert(ures.minable.results,{type="item",name="stone",amount=1,probability=.125})
		end
		if ures.minable.results and #ures.minable.results > yieldresults then yieldresults = #ures.minable.results end
		
		normal=nil
		temp = defined_richness[res.name]
		if temp then
			normal = temp.normal
			if normal then
				temp = temp.mimimum
				if temp then
					if temp > normal then ures.minimum = 1 end
					if temp > 1 then ures.minimum = ures.minimum/normal.normal end
				end
			else
				temp = temp.minimum
				if temp and temp > 1 then
					if temp>100 then temp=1 else temp = temp/100 end
				end
				ures.minimum = temp
			end
		end
		
		if not normal then
			normal = .1
			for _,peak in pairs(ures.autoplace.peaks) do
				if next(peak,"influence") then	--RSO hack, if peak is a constant, ignore it
					normal = normal + ((peak.influence or 1)+(peak.richness_influence or 0))/10
				end
			end
			if normal<0 then normal = -normal end
			normal = ures.autoplace.richness_base + floor(ures.autoplace.richness_multiplier*normal)
		end
		if ures.minimum then
			ures.minimum = ceil(ures.minimum*normal)
		else
			temp = log10(normal)/32
			if temp > .25 then temp = .25 end --autocalced vals max at 25% ()
			ures.minimum= ceil(normal * temp)
		end
		
		ures.normal = normal
		
		stages = #ures.stage_counts
		if stages>2 then
			local e = (normal*1.25 / ures.minimum)^(1/(stages-1))
			normal = normal * 1.2
			for i=1,stages do
				ures.stage_counts[i]=makenice(normal+2)
				normal = normal/e
			end
		end
		

		ures.infinite=not be_finite[res.name]--true
		
		table.insert(logger,string.format("%s(%i-%i): %i/%i(%0.2f%%) %s",ures.name,ures.autoplace.richness_base, ures.autoplace.richness_multiplier,
											ures.normal,ures.minimum, ures.minimum/ures.normal*100, serpent.line(ures.stage_counts)))
		
		ures.autoplace=nil

		table.insert(newres,ures)
--		table.insert(logger,"minable "..serpent.line(ures.minable))
	end
end

data.raw["mining-drill"]["underground-mining-drill"].storage_slots = yieldresults
--error(table.concat(logger,"\n"))

data:extend(newres)

--error(serpent.block(data.raw.resource["copper-ore"])..serpent.block(data.raw.resource["underground-copper-ore"]))
