require("prototypes.burner-drills")
require("prototypes.research")

--[[	Underground Mining configuration
All mods have access to the data object, and any elements not relevant to Factorio are ignored as it loads.
Factorio does not check excess data (that not in raw) for conformity to any standard.
Thus, mods (at load time) may communicate with one another through the data object.
Underground Mining has arbitrarily chosen "data.modcom" as the vehicle for mod communication,
and examines "data.modcom.underground_mining" for its configuration data which may be used by other mods
to help Underground Mining act consistently with their content, in the case that its default behavior is not
desired. The handling of underground resources is performed in data-updates.lua, so any resources or
settings declared beyond data.lua will not be recognized.

Underground Mining's settings for "pure" and "finite" consist of "flagged" content, a table whose keys
consist of the affected surface resource, and whose values are irrelevant (arbitrarily, the boolean "true");
for example,

	data.modcom.underground_mining.finite["coal"] = true

pure
	By default, underground resources have a slightly reduced chance to produce any resource they would
	normally (87.5%) and a small (12.5%) chance to generate stone (these odds are rolled separately by the
	Factorio engine, so any given production may produce both or neither). Flagging a resource here will
	disable this adjustment; the underground version of a resource will produce identically to the surface.
	
finite
	The underground version of this resource will not be infinite and will be depleted normally. Finite
	resources will have similar total values as their surface counterparts, roughly doubling the total
	resources available from an area (slightly more if "pure").

richness
	By default, Underground Mining recognizes resources by their autoplace information. If you have a resource
	that is generated by another method (by script, for example), and/or if you wish to override the method
	Underground Mining uses to generate richness characteristics for its underground version, add an entry for
	it here, using the following format:
	
		data.modcom.underground_mining.richness["iron-ore"]=
			{
			  normal = 1000,
			  minimum = 110,
			}
		
		OR
		
		data.modcom.underground_mining.richness["iron-ore"]=
			{
			  minimum = 0.5,
			}
	
	An underground resource at a value of normal will have 100% richness, and will drop over time to the defined
	minimum (the yield thereof will be equal to minimum/normal; in the first example, 11%). A minimum value
	supplied here that is less than 1 will be interpreted as a percentage.
	
	If normal is not defined (as in the second example), its value will be calculated normally by the mod; in
	this case the minimum value is always interpreted as a percentage. Undeclared minimum values will use the
	default formula, which is based on the magnitude of the normal and results in 10% for a normal value
	of 1200.
	
	When placed on the map at runtime, an infinite underground resource's beginning value is equal to, roughly,
	five times the amount of a typical nearby surface resource.

do_not_calculate
	Underground resources will not be generated for this resource. Use this when you do not want an underground
	resource to exist, or if you have already defined your own (named after its surface equivalent and prefixed
	with "underground-"; for example, "underground-iron-ore" or "underground-coal"). Because Underground Mining
	will not handle the definition of these resources, it is unnecessary to also use any of the above
	declarations.
--]]

if not data.modcom then data.modcom={} end
if not data.modcom.underground_mining then data.modcom.underground_mining={} end
local init = data.modcom.underground_mining

if not init.do_not_calculate then init.do_not_calculate = {} end
if not init.finite then init.finite = {} end
if not init.richness then init.richness = {} end
if not init.pure then init.pure = {} end

init.pure["stone"]=true
init.richness["stone"]={minimum=.2}
